@@ -1,4 +1,4 @@
-/* $Id: tif_print.c,v 1.19 2004/10/10 11:56:03 dron Exp $ */
+/* $Id: tif_print.c,v 1.25 2005/06/01 10:17:21 dron Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -107,10 +107,6 @@ TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
 		  (unsigned long) td->td_imagefullwidth,
 		  (unsigned long) td->td_imagefulllength);
  	}
- 	if (TIFFFieldSet(tif,FIELD_TEXTUREFORMAT))
-	  _TIFFprintAsciiTag(fd, "Texture Format", td->td_textureformat);
- 	if (TIFFFieldSet(tif,FIELD_WRAPMODES))
-	  _TIFFprintAsciiTag(fd, "Texture Wrap Modes", td->td_wrapmodes);
  	if (TIFFFieldSet(tif,FIELD_FOVCOT))
 	  fprintf(fd, "  Field of View Cotangent: %g\n", td->td_fovcot);
 	if (TIFFFieldSet(tif,FIELD_MATRIX_WORLDTOSCREEN)) {
@@ -282,8 +278,6 @@ TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
 	if (TIFFFieldSet(tif,FIELD_DOTRANGE))
 		fprintf(fd, "  Dot Range: %u-%u\n",
 		    td->td_dotrange[0], td->td_dotrange[1]);
-	if (TIFFFieldSet(tif,FIELD_TARGETPRINTER))
-		_TIFFprintAsciiTag(fd, "Target Printer", td->td_targetprinter);
 	if (TIFFFieldSet(tif,FIELD_THRESHHOLDING)) {
 		fprintf(fd, "  Thresholding: ");
 		switch (td->td_threshholding) {
@@ -354,22 +348,6 @@ TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
 	if (TIFFFieldSet(tif,FIELD_HALFTONEHINTS))
 		fprintf(fd, "  Halftone Hints: light %u dark %u\n",
 		    td->td_halftonehints[0], td->td_halftonehints[1]);
-	if (TIFFFieldSet(tif,FIELD_ARTIST))
-		_TIFFprintAsciiTag(fd, "Artist", td->td_artist);
-	if (TIFFFieldSet(tif,FIELD_DATETIME))
-		_TIFFprintAsciiTag(fd, "Date & Time", td->td_datetime);
-	if (TIFFFieldSet(tif,FIELD_HOSTCOMPUTER))
-		_TIFFprintAsciiTag(fd, "Host Computer", td->td_hostcomputer);
-	if (TIFFFieldSet(tif,FIELD_COPYRIGHT))
-		_TIFFprintAsciiTag(fd, "Copyright", td->td_copyright);
-	if (TIFFFieldSet(tif,FIELD_DOCUMENTNAME))
-		_TIFFprintAsciiTag(fd, "Document Name", td->td_documentname);
-	if (TIFFFieldSet(tif,FIELD_IMAGEDESCRIPTION))
-		_TIFFprintAsciiTag(fd, "Image Description", td->td_imagedescription);
-	if (TIFFFieldSet(tif,FIELD_MAKE))
-		_TIFFprintAsciiTag(fd, "Make", td->td_make);
-	if (TIFFFieldSet(tif,FIELD_MODEL))
-		_TIFFprintAsciiTag(fd, "Model", td->td_model);
 	if (TIFFFieldSet(tif,FIELD_ORIENTATION)) {
 		fprintf(fd, "  Orientation: ");
 		if (td->td_orientation < NORIENTNAMES)
@@ -412,8 +390,6 @@ TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
 			break;
 		}
 	}
-	if (TIFFFieldSet(tif,FIELD_PAGENAME))
-		_TIFFprintAsciiTag(fd, "Page Name", td->td_pagename);
 	if (TIFFFieldSet(tif,FIELD_PAGENUMBER))
 		fprintf(fd, "  Page Number: %u-%u\n",
 		    td->td_pagenumber[0], td->td_pagenumber[1]);
@@ -492,82 +468,92 @@ TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
             int  i;
             short count;
 
-            count = (short) TIFFGetTagListCount( tif );
-            for( i = 0; i < count; i++ )
-            {
-                ttag_t  tag = TIFFGetTagListEntry( tif, i );
-                const TIFFFieldInfo *fld;
+            count = (short) TIFFGetTagListCount(tif);
+            for(i = 0; i < count; i++) {
+                ttag_t  tag = TIFFGetTagListEntry(tif, i);
+                const TIFFFieldInfo *fip;
+                uint16 value_count;
+                int j, mem_alloc = 0;
+                void *raw_data;
 
-                fld = TIFFFieldWithTag( tif, tag );
-                if( fld == NULL )
-                    continue;
+                fip = TIFFFieldWithTag(tif, tag);
+                if(fip == NULL)
+			continue;
 
-                if( fld->field_passcount )
-                {
-                    short value_count;
-                    int j;
-                    void *raw_data;
-                    
-                    if( TIFFGetField( tif, tag, &value_count, &raw_data ) != 1 )
-                        continue;
+		if(fip->field_passcount) {
+			if(TIFFGetField(tif, tag, &value_count, &raw_data) != 1)
+				continue;
+		} else {
+			if (fip->field_readcount == TIFF_VARIABLE
+			    || fip->field_readcount == TIFF_VARIABLE2)
+				value_count = 1;
+			else if (fip->field_readcount == TIFF_SPP)
+				value_count = td->td_samplesperpixel;
+			else
+				value_count = fip->field_readcount;
+			if (fip->field_type == TIFF_ASCII
+			    || fip->field_readcount == TIFF_VARIABLE
+			    || fip->field_readcount == TIFF_VARIABLE2
+			    || fip->field_readcount == TIFF_SPP
+			    || value_count > 1) {
+				if(TIFFGetField(tif, tag, &raw_data) != 1)
+					continue;
+			} else {
+				raw_data = _TIFFmalloc(
+					_TIFFDataSize(fip->field_type)
+					* value_count);
+				mem_alloc = 1;
+				if(TIFFGetField(tif, tag, raw_data) != 1)
+					continue;
+			}
+		}
 
-                    fprintf(fd, "  %s: ", fld->field_name );
+		fprintf(fd, "  %s: ", fip->field_name);
 
-                    for( j = 0; j < value_count; j++ )
-                    {
-			if( fld->field_type == TIFF_BYTE )
-                            fprintf( fd, "%d",
-                                     (int) ((char *) raw_data)[j] );
-			else if( fld->field_type == TIFF_SHORT )
-                            fprintf( fd, "%d",
-                                     (int) ((unsigned short *) raw_data)[j] );
-			else if( fld->field_type == TIFF_SSHORT )
-                            fprintf( fd, "%d",
-                                     (int) ((short *) raw_data)[j] );
-                        else if( fld->field_type == TIFF_LONG )
-                            fprintf( fd, "%d",
-                                     (int) ((unsigned long *) raw_data)[j] );
-                        else if( fld->field_type == TIFF_SLONG )
-                            fprintf( fd, "%d",
-                                     (int) ((long *) raw_data)[j] );
-			else if( fld->field_type == TIFF_RATIONAL )
-			    fprintf( fd, "%f",
-				     ((float *) raw_data)[j] );
-			else if( fld->field_type == TIFF_SRATIONAL )
-			    fprintf( fd, "%f",
-				     ((float *) raw_data)[j] );
-                        else if( fld->field_type == TIFF_ASCII )
-                        {
-                            fprintf( fd, "%s",
-                                     (char *) raw_data );
-                            break;
-                        }
-                        else if( fld->field_type == TIFF_DOUBLE )
-                            fprintf( fd, "%f",
-                                     ((double *) raw_data)[j] );
-                        else if( fld->field_type == TIFF_FLOAT )
-                            fprintf( fd, "%f",
-                                     ((float *) raw_data)[j] );
-                        else
-                        {
-                            fprintf( fd,
-                                     "<unsupported data type in TIFFPrint>" );
-                            break;
-                        }
+		for(j = 0; j < value_count; j++) {
+		    if(fip->field_type == TIFF_BYTE)
+		        fprintf(fd, "%u",
+				(unsigned int) ((uint16 *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_UNDEFINED)
+		        fprintf(fd, "0x%x",
+				(unsigned int) ((unsigned char *) raw_data)[j]);		    else if(fip->field_type == TIFF_SBYTE)
+		        fprintf(fd, "%d", (int) ((uint16 *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_SHORT)
+		        fprintf(fd, "%u",
+		    	    (unsigned int)((unsigned short *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_SSHORT)
+		        fprintf(fd, "%d", (int)((short *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_LONG)
+		        fprintf(fd, "%lu",
+		    	    (unsigned long)((unsigned long *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_SLONG)
+		        fprintf(fd, "%ld", (long)((long *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_RATIONAL
+			    || fip->field_type == TIFF_SRATIONAL
+			    || fip->field_type == TIFF_FLOAT)
+		        fprintf(fd, "%f", ((float *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_IFD)
+		        fprintf(fd, "0x%x",
+		    	    (int)((unsigned long *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_ASCII) {
+		        fprintf(fd, "%s", (char *) raw_data);
+		        break;
+		    }
+		    else if(fip->field_type == TIFF_DOUBLE)
+		        fprintf(fd, "%f", ((double *) raw_data)[j]);
+		    else if(fip->field_type == TIFF_FLOAT)
+		        fprintf(fd, "%f", ((float *)raw_data)[j]);
+		    else {
+		        fprintf(fd, "<unsupported data type in TIFFPrint>");
+		        break;
+		    }
 
-                        if( j < value_count-1 )
-                            fprintf( fd, "," );
-                    }
-                    fprintf( fd, "\n" );
-                } 
-                else if( !fld->field_passcount
-                         && fld->field_type == TIFF_ASCII )
-                {
-                    char *data;
-                    
-                    if( TIFFGetField( tif, tag, &data ) )
-                        fprintf(fd, "  %s: %s\n", fld->field_name, data );
-                }
+		    if(j < value_count - 1)
+		        fprintf(fd, ",");
+		}
+		fprintf(fd, "\n");
+		if(mem_alloc)
+			_TIFFfree(raw_data);
             }
         }
         
